//*****************************************************************************
//
// weather.c - Internet Weather forcast client and web server.
//
// Copyright (c) 2007-2011 Texas Instruments Incorporated.  All rights reserved.
// Software License Agreement
// 
// Texas Instruments (TI) is supplying this software for use solely and
// exclusively on TI's microcontroller products. The software is owned by
// TI and/or its suppliers, and is protected under applicable copyright
// laws. You may not combine this software with "viral" open-source
// software in order to form a larger program.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND WITH ALL FAULTS.
// NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT
// NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. TI SHALL NOT, UNDER ANY
// CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES, FOR ANY REASON WHATSOEVER.
// 
//
//*****************************************************************************

#include "lwipopts.h"
#include <string.h>
#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"
#include "inc/hw_nvic.h"
#include "inc/hw_types.h"
#include "driverlib/flash.h"
#include "driverlib/fpu.h"
#include "driverlib/gpio.h"
#include "driverlib/interrupt.h"
#include "driverlib/sysctl.h"
#include "driverlib/systick.h"
#include "driverlib/timer.h"
#include "driverlib/rom.h"
#include "driverlib/rom_map.h"
#include "utils/uartstdio.h"
#include "utils/ustdlib.h"
#include "httpd.h"
#include "grlib/grlib.h"
#include "drivers/cfal96x64x16.h"
#include "io.h"
#include "cgifuncs.h"
#include "wlan.h" 
#include "evnt_handler.h"    // callback function declaration
#include "nvmem.h"
#include "socket.h"
#include "cc3000_common.h"
#include "netapp.h"
#include "spi.h"
#include "board_spi.h"
#include "images.h"

#include "drivers/buttons.h"

//*****************************************************************************
//
//! \addtogroup example_list
//! <h1>WiFi-based I/O Control and Weather Display client (weather)</h1>
//!
//! This example application demonstrates web-based I/O control using the
//! CC3000 SimpleLink WiFi module.  DHCP is used and required in this 
//! application.  There are two application examples inlcuded in this demo.  
//! One application is a web client and the other is a web server. 
//!
//! The web client application goes out to the web and retrieves an XML based
//! RSS weather feed from www.google.com for a given city.  The XML is parsed
//! for city name, current temperature and the day's forcasted high and low 
//! temperature in degress Fahrenheit.  The parsed information is then
//! graphically printed on the on-board OLED display.
//!
//! The Web server application is used to configure the web client application
//! city to retrieve the weather forcast from and the temperature units to
//! display.  The webserver serves up three static HTML pages and one dynamic
//! AJAX page. The dynamic page is the weather client configuration page 
//!
//! Note that the web server used by this example has been modified from the
//! example shipped with the basic lwIP package.  Additions include SSI and
//! CGI support along with the ability to have the server automatically insert
//! the HTTP headers rather than having these built in to the files in the
//! file system image.  This example was based off of the included
//! lwIP HTTP server and so includes much of the LwIP source and project 
//! structure.  However, the httpd application was completely re-written to
//! use the CC3000 driver library.
//!
//! This demo was written by Pacific Design Engineering LTD, 2011 (PDE LTD)
//! For more information about PDE refer to PDE's webpage at:
//! http://www.pde.com
//!
//! For more information regarding the CC3000 SimpleLink WiFi module refer to:
//! http://www.ti.com/CC3000
//!
//! For additional details on lwIP, refer to the lwIP web page at:
//! http://savannah.nongnu.org/projects/lwip/
//
//*****************************************************************************

//*****************************************************************************
//
// Defines for setting up the system clock.
//
//*****************************************************************************
#define SYSTICKHZ               100
#define SYSTICKMS               (1000 / SYSTICKHZ)

//*****************************************************************************
//
// The Processor Clock frequency in Hz
//
//*****************************************************************************
#define MCLK                    8000000

//*****************************************************************************
//
// Macro to convert from Fahrenheit to celcius
// Tc = (Tf-32) * (5/9)
//
//*****************************************************************************
#define FtoC(Tf)                (int)( (Tf - 32.0) * (5.0/9.0) ) 

//*****************************************************************************
//
// A set of flags.  The flag bits are defined as follows:
//
//     0 -> An indicator that the Fetch Weather timer elapsed or the web sever
//          finished serving up a page.
//     1 -> An indicator that the Smart Config has completed (due to first time
//          config process.)//     
//     2 -> An indicator that the CC3000 has made a connection to an AP.
//     3 -> An indicator that the CC3000 has been assigned an address by the AP
//          via DHCP.
//
//*****************************************************************************
#define FLAG_WEATHER_TICK             0
#define FLAG_CC3000_SMART_CONFIG_DONE 1
#define FLAG_CC3000_CONNECTED         2
#define FLAG_CC3000_RECEIVED_DHCP_IP  3
static volatile unsigned long g_ulFlags;

//*****************************************************************************
//
// The graphics context used by the application.
//
//*****************************************************************************
tContext g_sContext;

//*****************************************************************************
//
// Structure that defines an image in stored in memory
//
//*****************************************************************************
typedef struct
{
    const char *name;
    const unsigned char* icon;
} default_icon_filename;

//*****************************************************************************
//
// constant array of all the weather images stored in memory.  The images
// could have just been downloaded, which would be preferred to ensure
// they stay up to date, however there a GIF decoder doesn't exist yet.
//
//*****************************************************************************
const default_icon_filename g_psIconFilenames[] = {
  {"/ig/images/weather/sunny.gif", g_pucSunny },
  {"/ig/images/weather/mostly_sunny.gif", g_pucMostlySunny },
  {"/ig/images/weather/partly_cloudy.gif", g_pucPartlyCloudy },
  {"/ig/images/weather/mostly_cloudy.gif", g_pucMostlyCloudy },
  {"/ig/images/weather/chance_of_storm.gif", g_pucChanceOfStorm },
  {"/ig/images/weather/rain.gif", g_pucRain },
  {"/ig/images/weather/chance_of_rain.gif", g_pucChanceOfRain },
  {"/ig/images/weather/chance_of_snow.gif", g_pucChanceOfSnow },
  {"/ig/images/weather/cloudy.gif", g_pucCloudy },
  {"/ig/images/weather/mist.gif", g_pucMist },
  {"/ig/images/weather/storm.gif", g_pucStorm },
  {"/ig/images/weather/thunderstorm.gif", g_pucThunderstorm },
  {"/ig/images/weather/chance_of_tstorm.gif", g_pucThunderstorm },  
  {"/ig/images/weather/sleet.gif", g_pucSleet },
  {"/ig/images/weather/snow.gif", g_pucSnow },
  {"/ig/images/weather/icy.gif", g_pucIcy },
  {"/ig/images/weather/dust.gif", g_pucDust },
  {"/ig/images/weather/fog.gif", g_pucFog },
  {"/ig/images/weather/smoke.gif", g_pucSmoke },
  {"/ig/images/weather/haze.gif", g_pucHaze },
  {"/ig/images/weather/flurries.gif", g_pucFlurries },
};


#define NUM_DEFAULT_ICON_FILENAMES (sizeof(g_psIconFilenames) /                 \
                               sizeof(default_icon_filename))

//*****************************************************************************
//
// TODO
//
//*****************************************************************************
#define NETWORK_VIEW 0
#define WEATHER_VIEW 1
unsigned int g_view = NETWORK_VIEW;

//*****************************************************************************
//
// Position for the weather forcast Dsiplay
//
//*****************************************************************************
#define FORECAST_X     5
#define FORECAST_Y     5
#define MAX_FORECAST_X 96

//*****************************************************************************
//
// Position for the network IP address and AP SSID Dsiplay
//
//*****************************************************************************
#define NETWORK_X     5
#define NETWORK_Y     30
#define MAX_NETWORK_X 96

//*****************************************************************************
//
// Position for the status indicator shown while the IP address is being 
// determined.
//
//*****************************************************************************
#define STATUS_X         5
#define STATUS_Y         64-15
#define MAX_STATUS_X     96

//*****************************************************************************
//
// CC3000 SPI Interface PINs
//
//*****************************************************************************
#define SSI_CLK              GPIO_PIN_0
#define SSI_TX               GPIO_PIN_3
#define SSI_RX               GPIO_PIN_2
#define SSI_CS               GPIO_PIN_1

//*****************************************************************************
//
// Simple Config Prefix
//
//*****************************************************************************
char aucCC3000_prefix[] = {'P', 'D', 'E'};


tNetappIpconfigRetArgs g_tIpConfig;    
unsigned long g_ulIPAddress; 
unsigned long g_ulSeconds =0;

static WeatherForcast_t WeatherData;

//*****************************************************************************
//
// External Application references.
//
//*****************************************************************************
extern void httpd_init(void);
extern void SSIConfigure(unsigned long ulSSIFreq, unsigned long bForceGpioConfiguration, unsigned long uiReconfigureSysClock);

//*****************************************************************************
//
// CGI URI indices for each entry in the g_psConfigCGIURIs array.
//
//*****************************************************************************
#define CGI_INDEX_CONTROL       0
#define CGI_INDEX_TEXT          1

//*****************************************************************************
//
//! The number of individual CGI URIs that are configured for this system.
//
//*****************************************************************************
#define NUM_CONFIG_CGI_URIS     (sizeof(g_psConfigCGIURIs) / sizeof(tCGI))

//*****************************************************************************
//
//! The file sent back to the browser in cases where a parameter error is
//! detected by one of the CGI handlers.  This should only happen if someone
//! tries to access the CGI directly via the broswer command line and doesn't
//! enter all the required parameters alongside the URI.
//
//*****************************************************************************
#define PARAM_ERROR_RESPONSE    "/perror.htm"

#define JAVASCRIPT_HEADER                                                     \
    "<script type='text/javascript' language='JavaScript'><!--\n"
#define JAVASCRIPT_FOOTER                                                     \
    "//--></script>\n"

//*****************************************************************************
//
// The error routine that is called if the driver library encounters an error.
//
//*****************************************************************************
#ifdef DEBUG
void
__error__(char *pcFilename, unsigned long ulLine)
{
}
#endif

//*****************************************************************************
//
//! sendDriverPatch
//!
//! \param  pointer to the length
//!
//! \return none
//!
//! \brief  The function returns a pointer to the driver patch: since there is no patch yet - 
//!				it returns 0
//
//*****************************************************************************
const unsigned char *sendDriverPatch(unsigned long *Length)
{
	*Length = 0;
	return NULL;
}

//*****************************************************************************
//
//! sendBootLoaderPatch
//!
//! \param  pointer to the length
//!
//! \return none
//!
//! \brief  The function returns a pointer to the boot loader patch: since there is no patch yet -
//!				it returns 0
//
//*****************************************************************************
const unsigned char *sendBootLoaderPatch(unsigned long *Length)
{
	*Length = 0;
	return NULL;
}

//*****************************************************************************
//
//! sendWLFWPatch
//!
//! \param  pointer to the length
//!
//! \return none
//!
//! \brief  The function returns a pointer to the FW patch: since there is no patch yet - it returns 0
//
//*****************************************************************************

const unsigned char *sendWLFWPatch(unsigned long *Length)
{
	*Length = 0;
	return NULL;
}

//*****************************************************************************
//
//! ReadWlanInterruptPin
//!
//! \param  none
//!
//! \return none
//!
//! \brief  return wlan interrup pin
//
//*****************************************************************************

long ReadWlanInterruptPin(void)
{
  return MAP_GPIOPinRead(SPI_GPIO_IRQ_BASE, SPI_IRQ_PIN);
}

//*****************************************************************************
//
//! Enable waln IrQ pin
//!
//! \param  none
//!
//! \return none
//!
//! \brief  Nonr
//
//*****************************************************************************


void WlanInterruptEnable()
{
   MAP_GPIOPinIntEnable(SPI_GPIO_IRQ_BASE, SPI_IRQ_PIN);
}

//*****************************************************************************
//
//! Disable waln IrQ pin
//!
//! \param  none
//!
//! \return none
//!
//! \brief  Nonr
//
//*****************************************************************************


void WlanInterruptDisable()
{
  MAP_GPIOPinIntDisable(SPI_GPIO_IRQ_BASE, SPI_IRQ_PIN);
}


//*****************************************************************************
//
//! WriteWlanPin
//!
//! \param  new val
//!
//! \return none
//!
//! \brief  This functions enables and disables the CC3000 Radio
//
//*****************************************************************************

void WriteWlanPin( unsigned char val )
{
	if(val)
	{
  		GPIOPinWrite(SPI_GPIO_IRQ_BASE, SPI_EN_PIN, SPI_EN_PIN);
	}
	else
	{
  		MAP_GPIOPinWrite(SPI_GPIO_IRQ_BASE, SPI_EN_PIN, 0);
	}
}


//*****************************************************************************
//
//! DrawStatus
//!
//!  \param  pointer to char array to print to the status section of the display
//!
//!  \return none
//!
//!  \brief  This function draws text to area of the display defined by the 
//!          status section
//
//*****************************************************************************
void DrawStatus(char const *pucBuf)
{
    //
    // Disable processor interrupts.
    //
    ROM_IntMasterDisable();
    
    tRectangle sRect;
    //
    // Clear the status area.
    //
    sRect.sXMin = STATUS_X-5;
    sRect.sYMin = STATUS_Y;
    sRect.sXMax = MAX_STATUS_X;
    sRect.sYMax = STATUS_Y + 15;
    
    GrContextForegroundSet(&g_sContext, ClrBlack);
    GrRectFill(&g_sContext, &sRect);
    GrContextFontSet(&g_sContext, g_pFontCmss12);
    GrContextForegroundSet(&g_sContext, ClrLightGrey);
    GrStringDrawCentered(&g_sContext, pucBuf, -1,
                             GrContextDpyWidthGet(&g_sContext) / 2,
                             STATUS_Y + 5, false);
    //
    // Enable processor interrupts.
    //
    ROM_IntMasterEnable();
}

//*****************************************************************************
//
//! CC3000_UsynchCallback
//!
//! \param  Event type
//!
//! \return none
//!
//! \brief  The function handles asynchronous events that come from CC3000 device 
//!		  and operates the user LED to have an on-board indication
//
//*****************************************************************************
void CC3000_UsynchCallback(long lEventType, char *pcData, unsigned char ucLength)
{
	if (lEventType == HCI_EVNT_WLAN_ASYNC_SIMPLE_CONFIG_DONE)
	{
		//
        // flag that we have completed smart config
        //
        HWREGBITW(&g_ulFlags, FLAG_CC3000_SMART_CONFIG_DONE) = 1;
	}

	if (lEventType == HCI_EVNT_WLAN_UNSOL_CONNECT)
	{
        //
        // flag that we are connected
        //
        HWREGBITW(&g_ulFlags, FLAG_CC3000_CONNECTED) = 1;
        
        //
        // Turn on the user LED
        //
        io_set_led(1);
        
        DrawStatus("CC3000 Connected");
	}

	if (lEventType == HCI_EVNT_WLAN_UNSOL_DISCONNECT)
	{		
        HWREGBITW(&g_ulFlags, FLAG_CC3000_CONNECTED) = 0;
        //
        // Turn off the user LED
        //
        io_set_led(0);
        
        DrawStatus("CC3000 Not Connected");
	}
  
    if (lEventType == HCI_EVNT_WLAN_UNSOL_DHCP)
    {
        //
        // Should have an IP address now.
        // flag that we are connected
        //
        HWREGBITW(&g_ulFlags, FLAG_CC3000_RECEIVED_DHCP_IP) = 1;
    }
}

//*****************************************************************************
//
//! StartSmartConfig
//!
//!  \param  None
//!
//!  \return none
//!
//!  \brief  The function triggers a smart configuration process on CC3000.
//!			it exists upon completion of the process
//
//*****************************************************************************

void StartSmartConfig(void)
{
	//
	// Reset all the previous configuration
	//
	wlan_ioctl_set_connection_policy(0,0,0);	
  
	//
	// Trigger the Smart Config process
	//
	//
    // Turn on the user LED
    //
    io_set_led(1);
	wlan_first_time_config_set_prefix(aucCC3000_prefix);
    //
    // Turn off the user LED
    //
    io_set_led(0);
       
	//
	// Start the Smart Config process
	//
	wlan_first_time_config_start();
        
	//
    // Turn on the user LED
    //
    io_set_led(1);
	
	//
	// Wait for simple config finished
	// 
	while (!HWREGBITW(&g_ulFlags, FLAG_CC3000_SMART_CONFIG_DONE))
	{
		//
        // Turn off the user LED
        //
        io_set_led(0);
		SysCtlDelay(16500000);
		
		//
        // Turn on the user LED
        //
        io_set_led(1);                
        SysCtlDelay(16500000);
	}
    //
    // Turn on the user LED
    //
    io_set_led(1);
	//
	// Configure to connect automatically to the AP retrieved in the 
	// simple config process
	//
	wlan_ioctl_set_connection_policy(0, 0, 1);
	
	//
	// reset the CC3000
	// 
	wlan_stop();

	SysCtlDelay(100000);
	wlan_start(0);
  
    //
	// Mask out all non-required events from CC3000
	//
	wlan_set_event_mask(  HCI_EVNT_WLAN_KEEPALIVE 
                      | HCI_EVNT_WLAN_UNSOL_INIT
                      | HCI_EVNT_WLAN_ASYNC_PING_REPORT );
}


//*****************************************************************************
//
//! GPIOConfigure
//!
//! \param  none
//!
//! \return none
//!
//! \brief  Cofigure the GPIO
//
//*****************************************************************************
void
GPIOConfigure(void)
{	
	/* Configure the system peripheral bus that IRQ & EN pin are map to */
	MAP_SysCtlPeripheralEnable( SYSCTL_PERIPH_IRQ_PORT);
	//
	// Disable all the interrupts before configuring the lines
	//
	MAP_GPIOPinIntDisable(SPI_GPIO_IRQ_BASE, 0xFF);
    //
    // Cofigure WLAN_IRQ pin as input
    //
    MAP_GPIOPinTypeGPIOInput(SPI_GPIO_IRQ_BASE, SPI_IRQ_PIN);

    GPIOPadConfigSet(SPI_GPIO_IRQ_BASE, SPI_IRQ_PIN, GPIO_STRENGTH_2MA,
                   GPIO_PIN_TYPE_STD_WPU);     
    //
    // Setup the GPIO interrupt for this pin
    //
    MAP_GPIOIntTypeSet(SPI_GPIO_IRQ_BASE, SPI_IRQ_PIN, GPIO_FALLING_EDGE);

	  //
    // Disable WLAN chip
    //
    MAP_GPIOPinWrite(SPI_GPIO_IRQ_BASE, SPI_EN_PIN, 0);		
    MAP_GPIOPinTypeGPIOOutput(SPI_GPIO_IRQ_BASE, SPI_EN_PIN);
    GPIOPadConfigSet(SPI_GPIO_IRQ_BASE, SPI_EN_PIN, GPIO_STRENGTH_2MA,
                   GPIO_PIN_TYPE_STD_WPD);     
    
    MAP_GPIOPinWrite(SPI_GPIO_IRQ_BASE, SPI_EN_PIN, 0);	
    SysCtlDelay(MCLK/4);
	//
    // Disable WLAN CS with pull up Resistor
    //
	MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_SPI_PORT);	
    MAP_GPIOPinTypeGPIOOutput(SPI_PORT, SPI_CS_PIN);
    GPIOPadConfigSet(SPI_PORT, SPI_CS_PIN, GPIO_STRENGTH_2MA,
                   GPIO_PIN_TYPE_STD_WPU);     
    MAP_GPIOPinWrite(SPI_PORT, SPI_CS_PIN, SPI_CS_PIN);

    // 
    // Enable interrupt for WLAN_IRQ pin
    //
    MAP_GPIOPinIntEnable(SPI_GPIO_IRQ_BASE, SPI_IRQ_PIN);
	// 
	// Clear interrupt status
	//
	SpiCleanGPIOISR();
	
	MAP_IntEnable(INT_GPIO_SPI);
}

//*****************************************************************************
//
//! This function: CC3000InitSpi
//!
//!  \param  buffer
//!
//!  \return none
//!
//!  \brief  initializes an SPI interface
//
//*****************************************************************************

int CC3000InitSpi(void)
{
	//
	// Update the configuration in the SPI driver
	//
	SpiConfigureHwMapping();

    //
	//Configure SPI GPIO Related Pins 
    //
	GPIOConfigure();
    //
	// otherwise will receive redundant interrupts
    //
    SSIConfigure(1000000, 1, 50000000);
    
	return(ESUCCESS);
}

//*****************************************************************************
//
//! initCC3000Driver
//!
//!  \param  None
//!
//!  \return none
//!
//!  \brief  The function initializes a CC3000 device and triggers it to start operation
//
//*****************************************************************************
int
initCC3000Driver(void)
{	
    //
    //init all layers
    //
    CC3000InitSpi();
    
    //
    // Enable processor interrupts
    //
    ROM_IntMasterEnable(); 
	  //
	  // WLAN On API Implementation
	  //
	  wlan_init( CC3000_UsynchCallback, (tFWPatches )sendWLFWPatch, (tDriverPatches )sendDriverPatch, (tBootLoaderPatches)sendBootLoaderPatch, ReadWlanInterruptPin, WlanInterruptEnable, WlanInterruptDisable, WriteWlanPin);
	  //
	  // Trigger a WLAN device
	  //
	  wlan_start(0);	
  
    //
    // Setup DHCP
    //
    unsigned char pucSubnetMask[4], pucIP_Addr[4], pucIP_DefaultGWAddr[4],
            pucDNS[4];
    memset(pucSubnetMask,       0, sizeof(pucSubnetMask));
    memset(pucIP_Addr,          0, sizeof(pucIP_Addr));
    memset(pucIP_DefaultGWAddr, 0, sizeof(pucIP_DefaultGWAddr));
    memset(pucDNS,              0, sizeof(pucDNS));
    
    netapp_dhcp((unsigned long *) pucIP_Addr, (unsigned long *) pucSubnetMask,
            (unsigned long *) pucIP_DefaultGWAddr, (unsigned long *) pucDNS);
    //
    // Mask out all non-required events from CC3000
    //
    wlan_set_event_mask(  HCI_EVNT_WLAN_KEEPALIVE 
                      | HCI_EVNT_WLAN_UNSOL_INIT
                      | HCI_EVNT_WLAN_ASYNC_PING_REPORT );
    
    SysCtlDelay(100000);
		
    return(0);
}

//*****************************************************************************
//
//! initCC3000
//!
//!  \param  None
//!
//!  \return none
//!
//!  \brief  This function initializes the CC3000 and establishes 
//!			a connection to an AP, then opens a port.  The status of
//!     this function is returned as a CC3000Status_t enum
//
//*****************************************************************************
void initCC3000( void )
{
    unsigned char i;
    
    //
    //  Start the CC3000
    //
    initCC3000Driver(); 
    
    DrawStatus("Configuring CC3000");
    //
    // If the Select button is pressed then the user wants to delete the policy 
    // so they can connect to a different AP most likely.
    //
    if(ButtonsPoll(0, 0) & SELECT_BUTTON)
    {
        //
        // Reset all the previous configuration
        //
        wlan_ioctl_set_connection_policy(0,0,0);	
        wlan_ioctl_del_profile(0);
        wlan_ioctl_del_profile(1);
        wlan_ioctl_del_profile(2);
        //
        // reset the CC3000
        // 
        wlan_stop();      
        SysCtlDelay(MCLK/4);
        DrawStatus("Deleting Policies");
        initCC3000Driver();
    }
#ifdef SMART_CONFIG_EN      
    //
    // Wait 12 seconds to see if we connect
    //    
    DrawStatus("Trying to Connect");
    for(i = 0; (i < 24) &&
       (!HWREGBITW(&g_ulFlags, FLAG_CC3000_CONNECTED)); i++)
    {
        SysCtlDelay(MCLK/4);
        //
        // Turn on the user LED
        //
        io_set_led(1);
        SysCtlDelay(MCLK/4);
        //
        // Turn off the user LED
        //
        io_set_led(0);
    }
#endif
    //
    // test connection status
    // if we didn't get a connection to an AP then we can try connecting.
    // to the default AP SSID that ships with the demo TP-LINK
    //    
    if(!HWREGBITW(&g_ulFlags, FLAG_CC3000_CONNECTED))
    {
        wlan_connect(0, "TP-LINK", 0x07, NULL, NULL, 0);
        //
        // Wait 3 seconds to see if we connect this time
        //    
        for(i = 0; (i < 15) &&
           (!HWREGBITW(&g_ulFlags, FLAG_CC3000_CONNECTED)); i++)
        {
            SysCtlDelay(MCLK/4);
            //
            // Turn on the user LED
            //
            io_set_led(1);
            SysCtlDelay(MCLK/4);
            //
            // Turn off the user LED
            //
            io_set_led(0);
        }
    }
    //
    // test connection status again
    // if we didn't get a connection to the default AP then we can try the 
    // smart config process which listens for the secret prefix.
    // In this demo the prefix is:
    // char aucCC3000_prefix[] = {'P', 'D', 'E'};
    if(!HWREGBITW(&g_ulFlags, FLAG_CC3000_CONNECTED))
    {
        //
        // Try Connect to the AP using smart config
        //
#ifdef SMART_CONFIG_EN
        DrawStatus("Trying FTC");
        StartSmartConfig();   
        DrawStatus("FTC Done");           
#else
      // or, direct connect
#if   defined(SECURITY_WEP)
		wlan_connect(1, "TP-LINK", 0x07,
					NULL, "password", sizeof("password"));
#elif  defined(SECURITY_WPA)
		wlan_connect(2, "TP-LINK", 0x07,
					NULL, "password", sizeof("password"));
#elif  defined(SECURITY_WPA2)
		wlan_connect(3, "TP-LINK", 0x07,
					NULL, "password", sizeof("password"));
#else // no security
		wlan_connect(0, "TP-LINK", 0x07, NULL, NULL, 0);
#endif            
        
#endif      
        //
        // Wait 3 seconds to see if we connect this time
        //    
        for(i = 0; (i < 15) &&
           (!HWREGBITW(&g_ulFlags, FLAG_CC3000_CONNECTED)); i++)
        {
            SysCtlDelay(MCLK/4);
            //
            // Turn on the user LED
            //
            io_set_led(1);
            SysCtlDelay(MCLK/4);
            //
            // Turn off the user LED
            //
            io_set_led(0);
        }
    }
  
    //
    // Change the default inactivity timeout from 60s to the minimum
    // 20 seconds.  Resolution is 10 seconds
    //
    unsigned long DhcpTimeout, ArpTimeout, KeepAliveTimeout, InactivityTimeout;
    DhcpTimeout = 6;
    ArpTimeout = 6;
    KeepAliveTimeout = 6;
    InactivityTimeout = 2;
    netapp_timeout_values(&DhcpTimeout, &ArpTimeout, \
                       &KeepAliveTimeout, &InactivityTimeout);
    
    //
    // There is only one LED used for multiple indicators.
    // Set the state of the LED based on the connection to the 
    // AP before leaving.
    //
    if(HWREGBITW(&g_ulFlags, FLAG_CC3000_CONNECTED))
    {
      DrawStatus("CC3000 Connected");
      io_set_led(1); 
    }
    else
    {
      DrawStatus("CC3000 Not Connected");
      io_set_led(0); 
    }  
}

//*****************************************************************************
//
//! DisplayNoInternet
//!
//!  \param  none
//!
//!  \return none
//!
//!  \brief  This function notifies the user that there is no internet 
//           connection by displaying a message on the LCD.
//
//*****************************************************************************
void
DisplayNoInternet(void)
{
    //
    // Disable processor interrupts.
    //
    ROM_IntMasterDisable();
    tRectangle sRect;
    //
    // Clear the whole screen
    //
    sRect.sXMin = 0;
    sRect.sYMin = 0;
    sRect.sXMax = GrContextDpyWidthGet(&g_sContext)-1;
    sRect.sYMax = GrContextDpyHeightGet(&g_sContext)-1;
    GrContextForegroundSet(&g_sContext, ClrBlack);
    GrRectFill(&g_sContext, &sRect);
    
    //
    // Print Message
    //
    GrContextForegroundSet(&g_sContext, ClrYellow);
    GrContextFontSet(&g_sContext, g_pFontCmss12);
    GrStringDrawCentered(&g_sContext, "No Internet", -1,
                             GrContextDpyWidthGet(&g_sContext) / 2,
                             FORECAST_Y + 18, false);
    
    GrStringDrawCentered(&g_sContext, "Connection!", -1,
                             GrContextDpyWidthGet(&g_sContext) / 2,
                             FORECAST_Y + 32, false);
    
    //
    // Enable processor interrupts.
    //
    ROM_IntMasterEnable();
    
}
//*****************************************************************************
//
//! DisplayWeatherForcast
//!
//!  \param  pointer to structure with wether data to drisplay
//!
//!  \return none
//!
//!  \brief  This function the weather forcast information on the display
//!          defined by the weather display area.
//
//*****************************************************************************
void
DisplayWeatherForcast(WeatherForcast_t *ptWeatherData)
{
    char pucBuf[10];
    //
    // Disable processor interrupts.
    //
    ROM_IntMasterDisable();
    tRectangle sRect;
    //
    // Clear the weather area.
    //
    sRect.sXMin = FORECAST_X - 5;
    sRect.sYMin = FORECAST_Y - 5;
    sRect.sXMax = MAX_FORECAST_X;
    sRect.sYMax = 64 - 18;
    GrContextForegroundSet(&g_sContext, ClrBlack);
    GrRectFill(&g_sContext, &sRect);
    
    //
    // Display Weather
    //
    GrContextForegroundSet(&g_sContext, ClrWhite);
    GrContextFontSet(&g_sContext, g_pFontCmss12);
    
    //
    // get the temperature display unit type and draw the current temperature
    //
    io_get_units_string(pucBuf);
    if(!strcmp("Fahrenheit", pucBuf))
    {
        usprintf(pucBuf, "%dF", ptWeatherData->iTemperatureCurrent);
    }
    else
    {      
        usprintf(pucBuf, "%dC", FtoC(ptWeatherData->iTemperatureCurrent));
    }    
    GrStringDraw(&g_sContext,pucBuf , -1, FORECAST_X, FORECAST_Y - 4, false);
    
    GrContextFontSet(&g_sContext, g_pFontCmss12);
    
    //
    // get the temperature display unit type and draw the forcasted day's
    // highs and lows
    //
    io_get_units_string(pucBuf);
    if(!strcmp("Fahrenheit", pucBuf))
    {
        usprintf(pucBuf, "%dF/%dF", ptWeatherData->iTemperatureHigh, 
             ptWeatherData->iTemperatureLow);
    }
    else
    {      
        usprintf(pucBuf, "%dC/%dC", FtoC(ptWeatherData->iTemperatureHigh), 
             FtoC(ptWeatherData->iTemperatureLow));
    }
    
    GrStringDraw(&g_sContext,pucBuf , 10, FORECAST_X, FORECAST_Y + 8, false);
    //
    // use current forecast image url to search for image and then 
    // Draw Weather Icon 
    //
    int loop;
    for(loop = 0; loop < NUM_DEFAULT_ICON_FILENAMES; loop++) 
    {
      if(!strcmp(ptWeatherData->pcIconUrl, g_psIconFilenames[loop].name))
      {
        GrImageDraw(&g_sContext, g_psIconFilenames[loop].icon, 
                     56, FORECAST_Y-5);
        
         break;
      }
    }
    
    GrContextFontSet(&g_sContext, g_pFontCmss12i);
    GrStringDraw(&g_sContext, ptWeatherData->pcCityName, -1, FORECAST_X,
                             FORECAST_Y + 32, false);
    
    g_view = WEATHER_VIEW;
    //
    // Enable processor interrupts.
    //
    ROM_IntMasterEnable();
}


//*****************************************************************************
//
// Display an IP Address.
//
//*****************************************************************************
void
DisplayIPAddress(unsigned long ipaddr, unsigned long ulRow,
                 unsigned long ulCol)
{
    char pucBuf[16];
    unsigned char *pucTemp = (unsigned char *)&ipaddr;

    //
    // Convert the IP Address into a string.
    //
    usprintf(pucBuf, "%d.%d.%d.%d", pucTemp[3], pucTemp[2], pucTemp[1],
             pucTemp[0]);

    //
    // Display the string.
    //
    GrContextFontSet(&g_sContext, g_pFontCmss12);
    
    GrStringDrawCentered(&g_sContext, pucBuf, -1, 
             ulRow, ulCol, true);
}

//*****************************************************************************
//
//! DisplayNetworkPage
//!
//!  \param  pointer to structure holding ipconfig information
//!
//!  \return none
//!
//!  \brief  This function draws text of the assigned IP address and the SSID
//!          of the AP on the display in the area defined by the 
//!          network section
//
//*****************************************************************************
void
DisplayNetworkPage(tNetappIpconfigRetArgs *ptNetworkArgs)
{
    //
    // Disable processor interrupts.
    //
    ROM_IntMasterDisable();
    tRectangle sRect;
    //
    // Clear the status area.
    //
    sRect.sXMin = NETWORK_X - 5;
    sRect.sYMin = NETWORK_Y - 5;
    sRect.sXMax = MAX_NETWORK_X;
    sRect.sYMax = NETWORK_Y + 15;
    GrContextForegroundSet(&g_sContext, ClrBlack);
    GrRectFill(&g_sContext, &sRect);
    
    GrContextForegroundSet(&g_sContext, ClrWhite);
    GrContextFontSet(&g_sContext, g_pFontCmss12);    
    
    DisplayIPAddress(g_ulIPAddress, GrContextDpyWidthGet(&g_sContext) / 2, 
                     NETWORK_Y - 6);   
    //
    // Display the SSID string.
    //
    GrContextFontSet(&g_sContext, g_pFontCmss12);
    GrStringDrawCentered(&g_sContext, (char*)ptNetworkArgs->uaSSID, -1, 
             GrContextDpyWidthGet(&g_sContext) / 2, NETWORK_Y + 7, true);
    g_view = NETWORK_VIEW;
    //
    // Enable processor interrupts.
    //
    ROM_IntMasterEnable();
}

//*****************************************************************************
//
//! SysTickIntHandler
//!
//!  \param  none
//!
//!  \return none
//!
//!  \brief  This is the systick interrupt handler and is used to service the 
//!          CC3000 hci_unsolicited_event_handler(), a time base to update the 
//!          weather forcast once per minute and to display the network 
//!          ipconfig information if the select button is pressed.
//!
//!          10ms Tick intervals
//
//*****************************************************************************
void
SysTickIntHandler(void)
{
    static unsigned long ulCC3000TickCount = 0, ulWeatherTickCount = 0;
    static unsigned int NetworkPageDisplayed = false;
    //
    // Increment the CC3000 tick counter.
    //
    ulCC3000TickCount++;
    //Process after 500 ms
    if(ulCC3000TickCount >= 50)
    {
        //
        // Handle any un-solicited event if required - the function shall be triggered 
        // few times in a second
        //
        if(HWREGBITW(&g_ulFlags, FLAG_CC3000_CONNECTED))
        {
            hci_unsolicited_event_handler();
        }
        ulCC3000TickCount = 1;
    }
    
    //
    // Increment the Weather tick counter.
    //
    ulWeatherTickCount++;
    //
    //Process after (1 minute)
    //
    if(ulWeatherTickCount >= 6000)
    {
        //
        // Set the flag to update the Weather forcast
        //
        HWREGBITW(&g_ulFlags, FLAG_WEATHER_TICK) = 1;
        ulWeatherTickCount = 1;
    }
    //
    // Display the network information if the select button is pressed
    // and the ip address has been assigned
    //
    if(g_ulIPAddress != NULL)
    {
        if(ButtonsPoll(0, 0) & SELECT_BUTTON)
        {
            if(NetworkPageDisplayed == false)
            {
                tRectangle sRect;
                //
                // blank the screen
                //
                sRect.sXMin = 0;
                sRect.sYMin = 0;
                sRect.sXMax = GrContextDpyWidthGet(&g_sContext) - 1;
                sRect.sYMax = GrContextDpyHeightGet(&g_sContext) - 1;
                GrContextForegroundSet(&g_sContext, ClrBlack);
                GrRectFill(&g_sContext, &sRect);
                DisplayNetworkPage(&g_tIpConfig);
                NetworkPageDisplayed = true;
            }
        }
        else if(NetworkPageDisplayed == true)
        {
          NetworkPageDisplayed = false;
          DisplayWeatherForcast(&WeatherData); 
        }
    }
}
//*****************************************************************************
//
// This is the handler for the User Button.  
//
//*****************************************************************************
void
ButtonInterruptHandler(void)
{
    DisplayNetworkPage(&g_tIpConfig);
}

//*****************************************************************************
//
//! main
//!
//!  \param  none
//!
//!  \return none
//!
//!  \brief  This main routine intializes all devices and runs in an infinite 
//! while loop. This application spends most of it's time in the Web server 
//! waiting for a client to connect. The Webserver returns from it's call after 
//! a timeout of were no clients have connected in the last 20 seconds or after 
//! serving up a web page.  after one minute or when the weather client  
//! configuration page is updated, the weather client is started and retrieves 
//! and displays the weather forcast.
//
//*****************************************************************************
int
main(void)
{
	int i;
    tRectangle sRect;
    
    //
    //  Board Initialization start
    //
    //
    // The FPU should be enabled because some compilers will use floating-
    // point registers, even for non-floating-point code.  If the FPU is not
    // enabled this will cause a fault.  This also ensures that floating-
    // point operations could be added to this application and would work
    // correctly and use the hardware floating-point unit.  Finally, lazy
    // stacking is enabled for interrupt handlers.  This allows floating-
    // point instructions to be used within interrupt handlers, but at the
    // expense of extra stack usage.
    //
    FPUEnable();
    FPULazyStackingEnable();	

    //
    // Set the system clock to run at 50MHz from the PLL.
    //
    ROM_SysCtlClockSet(SYSCTL_SYSDIV_4 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN |
                       SYSCTL_XTAL_16MHZ);
  
    i = sizeof(hci_cmnd_hdr_t);
    //
    // Enable all GPIO banks.
    //
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB);
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOC);
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOD);
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOE);
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOF);
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOG);
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOH);
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOJ);
    
    // Set CC3000 En Pin High while booting up
    // We will then reset it before initialization.
    
    MAP_GPIOPinWrite(SPI_GPIO_IRQ_BASE, SPI_EN_PIN, SPI_EN_PIN);		
    MAP_GPIOPinTypeGPIOOutput(SPI_GPIO_IRQ_BASE, SPI_EN_PIN);
    GPIOPadConfigSet(SPI_GPIO_IRQ_BASE, SPI_EN_PIN, GPIO_STRENGTH_2MA,
                   GPIO_PIN_TYPE_STD_WPD);     
    
    MAP_GPIOPinWrite(SPI_GPIO_IRQ_BASE, SPI_EN_PIN, SPI_EN_PIN);	
    SysCtlDelay(MCLK/4);
    
    //
    // Initialize IO controls
    //
    io_init();   
    
    //
    // Initialize the buttons driver.
    //
    ButtonsInit();
    
    //
    // Set GPIO A0 and A1 as UART.
    //
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);

    //
    // Initialize the UART as a console for text I/O.
    //
    UARTStdioInit(0);
    
    UARTprintf("CC3000 SimpleLink Wifi Weather Demo Application\n");
       
    //
    // ensure that the CC3000 IRQ pin has the highest priority
    //
    IntPriorityGroupingSet(4);
    //
    // 0 is the highest
    //
    IntPrioritySet(INT_SSI0, ((0 << (NUM_PRIORITY - NUM_PRIORITY_BITS)) & 0xE0)  ); 
    IntPrioritySet(INT_GPIOA, ((1 << (NUM_PRIORITY - NUM_PRIORITY_BITS)) & 0xE0)  ); 

    //
    // Initialize the display driver.
    //
    CFAL96x64x16Init();

    //
    // Initialize the graphics context.
    //
    GrContextInit(&g_sContext, &g_sCFAL96x64x16);

    //
    // Draw Splash Screen
    //
    const unsigned char *pucSplashLogo = g_pucPdeLogoImage;

    //
    // Draw the PDE logo on the display.  Use an animation effect where the
    // logo will "slide" onto the screen.  
    //
    long uX;
    for(uX = 0; uX < 96; uX++)
    {
        GrImageDraw(&g_sContext, pucSplashLogo, 95 - uX, 0);
    }    
    
    // 2 seconds
    SysCtlDelay(MCLK*2);
    
    //
    // Fill the entire display with a black rectangle, to clear it.
    //
    sRect.sXMin = 0;
    sRect.sYMin = 0;
    sRect.sXMax = 95;
    sRect.sYMax = 63;
    GrContextForegroundSet(&g_sContext, ClrBlack);
    GrRectFill(&g_sContext, &sRect);
    
    //
    // Fill the top 24 rows of the screen with blue to create the banner.
    //
    sRect.sXMin = 0;
    sRect.sYMin = 0;
    sRect.sXMax = GrContextDpyWidthGet(&g_sContext) - 1;
    sRect.sYMax = 15;
    GrContextForegroundSet(&g_sContext, ClrDarkBlue);
    GrRectFill(&g_sContext, &sRect);

    //
    // Put a white box around the banner.
    //
    GrContextForegroundSet(&g_sContext, ClrWhite);
    GrRectDraw(&g_sContext, &sRect);

    //
    // Put the application name in the middle of the banner.
    //
    GrContextFontSet(&g_sContext, g_pFontCm12);
    GrStringDrawCentered(&g_sContext, "WiFi Demo", -1,
                         GrContextDpyWidthGet(&g_sContext) / 2, 7, 0);
    
    //
    // Tell the user what we are doing just now.
    //
    DrawStatus("Initializing CC3000");
    
    //
    // Initialize the CC3000.
    //
    initCC3000();  
    
    //
    // Tell the user what we are doing just now.
    //
    DrawStatus("Waiting for IP");
    
    //
    // Configure SysTick for a periodic interrupt.
    //
    ROM_SysTickPeriodSet(ROM_SysCtlClockGet() / SYSTICKHZ);
    ROM_SysTickEnable();
    ROM_SysTickIntEnable();
        
    //
    //
    // update weather immediately after connecting to AP
    //
    HWREGBITW(&g_ulFlags, FLAG_WEATHER_TICK) = 1;
    
    //
    // Loop forever, processing the on-screen animation.  All other work is
    // done in the interrupt handlers.
    //   
    while(1)
    {
        //
        // Print some white space on every loop.  Since Webserver blocks
        // this loop doesn't run very often.
        //
        UARTprintf("\n---------------------------------------------------\n\n");
        
        //
        // Get Connection details when connected
        //
        if(HWREGBITW(&g_ulFlags, FLAG_CC3000_CONNECTED) &&
           HWREGBITW(&g_ulFlags, FLAG_CC3000_RECEIVED_DHCP_IP) )
        {                  
            if(g_ulIPAddress == NULL)
            {
                //
                // Get ip Address
                //
                netapp_ipconfig(&g_tIpConfig);
                memcpy(&g_ulIPAddress,&(g_tIpConfig.aucIP), 4);
                DisplayNetworkPage(&g_tIpConfig);
            }
        
            //
            // weather client
            //
            if(HWREGBITW(&g_ulFlags, FLAG_WEATHER_TICK))
            {
                //
                // Tell the user what we are doing just now.
                //
                DrawStatus("Get Weather");  
                
                //
                // Clear the flag now that we have seen it.
                //
                HWREGBITW(&g_ulFlags, FLAG_WEATHER_TICK) = 0;
                
                //
                // Fetch Weather
                // Try up to 3 times   
                //
                long lerr;
                for(i = 0; i < 3; i++)
                {
                    lerr = http_get_weather_for_city(g_pucWeatherCity, 
                                                             &WeatherData);
                    if(ESUCCESS == lerr)
                    {
                        DisplayWeatherForcast(&WeatherData);     
                        break;
                    }
                    else if(-74 == lerr)
                    {
                        // Suspect there is no internet activity
                        DisplayNoInternet();
                        break;
                    }
                }
            }   
        
            // 
            // Web server          
            //
            // Tell the user what we are doing just now.
            //
            DrawStatus("Web Server");
            httpd_init();
            
            //
            // If the webserver updates the weather 
            // application configuration then the weather forcast
            // is fetched immediatly after return.  
            //          
            if(io_get_weather_application_update())
            {
                //
                // Clear the weather updated Flag
                //
                io_set_weather_application_update(false);
                //
                // Set the flag to update the Weather forcast
                //
                HWREGBITW(&g_ulFlags, FLAG_WEATHER_TICK) = 1;
            }
        }       
    }
}
