/*
 * Copyright (c) 2001-2003 Swedish Institute of Computer Science.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * This file is part of the lwIP TCP/IP stack.
 *
 * Author: Adam Dunkels <adam@sics.se>
 *
 */

/* This file is modified from the original version (httpd.c) as shipped with
 * lwIP version 1.3.2. Changes have been made to allow support for a
 * rudimentary server-side-include facility which will replace tags of the form
 * <!--#tag--> in any file whose extension is .shtml, .shtm or .ssi with
 * strings provided by an include handler whose pointer is provided to the
 * module via function http_set_ssi_handler(). Additionally, a simple common
 * gateway interface (CGI) handling mechanism has been added to allow clients
 * to hook functions to particular request URIs.
 *
 * To enable SSI support, define label INCLUDE_HTTPD_SSI in lwipopts.h.
 * To enable CGI support, define label INCLUDE_HTTPD_CGI in lwipopts.h.
 *
 * By default, the server assumes that HTTP headers are already present in
 * each file stored in the file system.  By defining DYNAMIC_HTTP_HEADERS in
 * lwipopts.h, this behavior can be changed such that the server inserts the
 * headers automatically based on the extension of the file being served.  If
 * this mode is used, be careful to ensure that the file system image used
 * does not already contain the header information.
 *
 * File system images without headers can be created using the makefsfile
 * tool with the -h command line option.
 */

/*
 * Notes about valid SSI tags
 * --------------------------
 *
 * The following assumptions are made about tags used in SSI markers:
 *
 * 1. No tag may contain '-' or whitespace characters within the tag name.
 * 2. Whitespace is allowed between the tag leadin "<!--#" and the start of
 *    the tag name and between the tag name and the leadout string "-->".
 * 3. The maximum tag name length is MAX_TAG_NAME_LEN, currently 8 characters.
 *
 * Notes on CGI usage
 * ------------------
 *
 * The simple CGI support offered here works with GET method requests only
 * and can handle up to 16 parameters encoded into the URI. The handler
 * function may not write directly to the HTTP output but must return a
 * filename that the HTTP server will send to the browser as a response to
 * the incoming CGI request.
 *
 */
#include "driverlib/rom.h"
#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"
#include "inc/hw_nvic.h"
#include "inc/hw_types.h"
#include "driverlib/gpio.h"
#include "driverlib/sysctl.h"
#include "utils/uartstdio.h"
#include "lwipopts.h"
#include "httpd.h"
#include "fs.h"
#include "arch/cc.h"
#include "lwip/err.h"
#include "lwip/inet.h"
#include "io.h"

#include "wlan.h" 
#include "evnt_handler.h"    // callback function declaration
#include "nvmem.h"
#include "socket.h"
#include "cc3000_common.h"
#include "netapp.h"
#include "spi.h"
#include "board_spi.h"
#include <string.h>

#ifndef HTTPD_DEBUG
#define HTTPD_DEBUG         LWIP_DBG_OFF
#endif

#ifndef true
#define true ((u8_t)1)
#endif

#ifndef false
#define false ((u8_t)0)
#endif

typedef struct
{
    const char *name;
    u8_t shtml;
} default_filename;

const default_filename g_psDefaultFilenames[] = {
  {"/index.shtml", true },
  {"/index.ssi", true },
  {"/index.shtm", true },
  {"/index.html", false },
  {"/index.htm", false }
};

#define NUM_DEFAULT_FILENAMES (sizeof(g_psDefaultFilenames) /                 \
                               sizeof(default_filename))

#ifdef DYNAMIC_HTTP_HEADERS
/* The number of individual strings that comprise the headers sent before each
 * requested file.
 */
#define NUM_FILE_HDR_STRINGS 3
#endif

struct http_state {
  struct fs_file *handle;
  char *file;       /* Pointer to first unsent byte in buf. */
  char *buf;        /* File read buffer. */
  u32_t left;       /* Number of unsent bytes in buf. */
  int buf_len;      /* Size of file read buffer, buf. */
  u8_t retries;
#ifdef INCLUDE_HTTPD_CGI
  char *params[MAX_CGI_PARAMETERS]; /* Params extracted from the request URI */
  char *param_vals[MAX_CGI_PARAMETERS]; /* Values for each extracted param */
#endif
#ifdef DYNAMIC_HTTP_HEADERS
  const char *hdrs[NUM_FILE_HDR_STRINGS]; /* HTTP headers to be sent. */
  u16_t hdr_pos;     /* The position of the first unsent header byte in the
                        current string */
  u16_t hdr_index;   /* The index of the hdr string currently being sent. */
#endif
};

#ifdef INCLUDE_HTTPD_CGI
/* CGI handler information */
const tCGI *g_pCGIs = NULL;
int g_iNumCGIs = 0;
#endif /* INCLUDE_HTTPD_CGI */

#ifdef DYNAMIC_HTTP_HEADERS

#define TX_MTU          800
//*****************************************************************************
//
// HTTP header strings for various filename extensions.
//
//*****************************************************************************
typedef struct
{
    const char *pszExtension;
    unsigned long ulHeaderIndex;
} tHTTPHeader;

const char *g_psHTTPHeaderStrings[] =
{
 "Content-type: text/html\r\n\r\n",
 "Content-type: text/html\r\nExpires: Fri, 10 Apr 2008 14:00:00 GMT\r\n"    \
   "Pragma: no-cache\r\n\r\n",
 "Content-type: image/gif\r\n\r\n",
 "Content-type: image/png\r\n\r\n",
 "Content-type: image/jpeg\r\n\r\n",
 "Content-type: image/bmp\r\n\r\n",
 "Content-type: image/x-icon\r\n\r\n",
 "Content-type: application/octet-stream\r\n\r\n",
 "Content-type: application/x-javascript\r\n\r\n",
 "Content-type: audio/x-pn-realaudio\r\n\r\n",
 "Content-type: text/css\r\n\r\n",
 "Content-type: application/x-shockwave-flash\r\n\r\n",
 "Content-type: text/xml\r\n\r\n",
 "Content-type: text/plain\r\n\r\n",
 "HTTP/1.0 200 OK\r\n",
 "HTTP/1.0 404 File not found\r\n",
 "Server: lwIP/1.3.2 (http://www.sics.se/~adam/lwip/)\r\n",
 "\r\n<html><body><h2>404: The requested file cannot be found."             \
   "</h2></body></html>\r\n"
};

#define HTTP_HDR_HTML           0
#define HTTP_HDR_SSI            1
#define HTTP_HDR_GIF            2
#define HTTP_HDR_PNG            3
#define HTTP_HDR_JPG            4
#define HTTP_HDR_BMP            5
#define HTTP_HDR_ICO            6
#define HTTP_HDR_APP            7
#define HTTP_HDR_JS             8
#define HTTP_HDR_RA             9
#define HTTP_HDR_CSS            10
#define HTTP_HDR_SWF            11
#define HTTP_HDR_XML            12
#define HTTP_HDR_DEFAULT_TYPE   13
#define HTTP_HDR_OK             14
#define HTTP_HDR_NOT_FOUND      15
#define HTTP_HDR_SERVER         16
#define DEFAULT_404_HTML        17

tHTTPHeader g_psHTTPHeaders[] =
{
 { "html", HTTP_HDR_HTML},
 { "htm",  HTTP_HDR_HTML},
 { "shtml",HTTP_HDR_SSI},
 { "shtm", HTTP_HDR_SSI},
 { "ssi",  HTTP_HDR_SSI},
 { "gif",  HTTP_HDR_GIF},
 { "png",  HTTP_HDR_PNG},
 { "jpg",  HTTP_HDR_JPG},
 { "bmp",  HTTP_HDR_BMP},
 { "ico",  HTTP_HDR_ICO},
 { "class",HTTP_HDR_APP},
 { "cls",  HTTP_HDR_APP},
 { "js",   HTTP_HDR_JS},
 { "ram",  HTTP_HDR_RA},
 { "css",  HTTP_HDR_CSS},
 { "swf",  HTTP_HDR_SWF},
 { "xml",  HTTP_HDR_XML}
};

#define NUM_HTTP_HEADERS (sizeof(g_psHTTPHeaders) / sizeof(tHTTPHeader))

#endif

//
// Reception from the air, buffer - the max data length  + headers
//
#define RX_SPI_BUFFER_SIZE    SPI_BUFFER_SIZE + 100
char pcCC3000_Rx_Buffer[RX_SPI_BUFFER_SIZE];
char pcCC3000_Tx_Buffer[SPI_BUFFER_SIZE];

long ulServerSocket, ulServerDataSocket;
long ulServerSocketopen = false;

sockaddr tClientSocketAddr;
socklen_t tClientSocketAddrLen;
struct http_state g_hs;

static void http_accept(void);
/*-----------------------------------------------------------------------------------*/
static void
close_conn(void)
{
  // Close server socket
  closesocket(ulServerDataSocket);
  UARTprintf("Close Socket: %d.\n", ulServerDataSocket);
  //
  // we could listen for the next connection bur
  // we since accept is a blocking call just return
  // to process other applications
}
/*-----------------------------------------------------------------------------------*/
#ifdef INCLUDE_HTTPD_CGI
static int
extract_uri_parameters(struct http_state *hs, char *params)
{
  char *pair;
  char *equals;
  int loop;

  /* If we have no parameters at all, return immediately. */
  if(!params || (params[0] == '\0')) {
      return(0);
  }

  /* Get a pointer to our first parameter */
  pair = params;

  /*
   * Parse up to MAX_CGI_PARAMETERS from the passed string and ignore the
   * remainder (if any)
   */
  for(loop = 0; (loop < MAX_CGI_PARAMETERS) && pair; loop++) {

    /* Save the name of the parameter */
    hs->params[loop] = pair;

    /* Remember the start of this name=value pair */
    equals = pair;

    /* Find the start of the next name=value pair and replace the delimiter
     * with a 0 to terminate the previous pair string.
     */
    pair = strchr(pair, '&');
    if(pair) {
      *pair = '\0';
      pair++;
    } else {
       /* We didn't find a new parameter so find the end of the URI and
        * replace the space with a '\0'
        */
        pair = strchr(equals, ' ');
        if(pair) {
            *pair = '\0';
        }

        /* Revert to NULL so that we exit the loop as expected. */
        pair = NULL;
    }

    /* Now find the '=' in the previous pair, replace it with '\0' and save
     * the parameter value string.
     */
    equals = strchr(equals, '=');
    if(equals) {
      *equals = '\0';
      hs->param_vals[loop] = equals + 1;
    } else {
      hs->param_vals[loop] = NULL;
    }
  }

  return(loop);
}
#endif /* INCLUDE_HTTPD_CGI */

/*-----------------------------------------------------------------------------------*/

#ifdef DYNAMIC_HTTP_HEADERS
//*****************************************************************************
//
// Generate the relevant HTTP headers for the given filename and write
// them into the supplied buffer.  Returns true on success or false on failure.
//
//*****************************************************************************
static void
get_http_headers(struct http_state *pState, char *pszURI)
{
    int iLoop;
    char *pszWork;
    char *pszExt;
    char *pszVars;

    //
    // Ensure that we initialize the loop counter.
    //
    iLoop = 0;

    //
    //
    // In all cases, the second header we send is the server identification
    // so set it here.
    //
    pState->hdrs[1] = g_psHTTPHeaderStrings[HTTP_HDR_SERVER];

    // Is this a normal file or the special case we use to send back the
    // default "404: Page not found" response?
    //
    if(pszURI == NULL)
    {
        pState->hdrs[0] = g_psHTTPHeaderStrings[HTTP_HDR_NOT_FOUND];
        pState->hdrs[2] = g_psHTTPHeaderStrings[DEFAULT_404_HTML];

        //
        // Set up to send the first header string.
        //
        pState->hdr_index = 0;
        pState->hdr_pos = 0;
        return;
    }
    else
    {
        //
        // We are dealing with a particular filename. Look for one other
        // special case.  We assume that any filename with "404" in it must be
        // indicative of a 404 server error whereas all other files require
        // the 200 OK header.
        //
        if(strstr(pszURI, "404"))
        {
            pState->hdrs[0] = g_psHTTPHeaderStrings[HTTP_HDR_NOT_FOUND];
        }
        else
        {
            pState->hdrs[0] = g_psHTTPHeaderStrings[HTTP_HDR_OK];
        }

        //
        // Determine if the URI has any variables and, if so, temporarily remove
        // them.
        //
        pszVars = strchr(pszURI, '?');
        if(pszVars)
        {
            *pszVars = '\0';
        }

        //
        // Get a pointer to the file extension.  We find this by looking for the
        // last occurrence of "." in the filename passed.
        //
        pszExt = NULL;
        pszWork = strchr(pszURI, '.');
        while(pszWork)
        {
            pszExt = pszWork + 1;
            pszWork = strchr(pszExt, '.');
        }

        //
        // Now determine the content type and add the relevant header for that.
        //
        for(iLoop = 0; (iLoop < NUM_HTTP_HEADERS) && pszExt; iLoop++)
        {
            //
            // Have we found a matching extension?
            //
            if(!strcmp(g_psHTTPHeaders[iLoop].pszExtension, pszExt))
            {
                pState->hdrs[2] =
                  g_psHTTPHeaderStrings[g_psHTTPHeaders[iLoop].ulHeaderIndex];
                break;
            }
        }

        //
        // Reinstate the parameter marker if there was one in the original
        // URI.
        //
        if(pszVars)
        {
            *pszVars = '?';
        }
    }

    //
    // Does the URL passed have any file extension?  If not, we assume it
    // is a special-case URL used for control state notification and we do
    // not send any HTTP headers with the response.
    //
    if(!pszExt)
    {
        //
        // Force the header index to a value indicating that all headers
        // have already been sent.
        //
        pState->hdr_index = NUM_FILE_HDR_STRINGS;
    }
    else
    {
        //
        // Did we find a matching extension?
        //
        if(iLoop == NUM_HTTP_HEADERS)
        {
            //
            // No - use the default, plain text file type.
            //
            pState->hdrs[2] = g_psHTTPHeaderStrings[HTTP_HDR_DEFAULT_TYPE];
        }

        //
        // Set up to send the first header string.
        //
        pState->hdr_index = 0;
        pState->hdr_pos = 0;
    }
}
#endif
/*-----------------------------------------------------------------------------------*/
static int
send_data(void)
{
  static char testBuff[TX_MTU];
  static unsigned char testIdx = 0;
  static u16_t len, rcvdlen;
  static  u8_t data_to_send = false;
#ifdef DYNAMIC_HTTP_HEADERS
  static  u16_t hdrlen, sendlen;
  struct http_state *phs;
  
  // get pointer to file system handle
  phs = &g_hs;

  /* If the file handle is NULL state structure pointer, ignore the call. */
  if(!phs) {
      return(-1);
  }

  testIdx = 0;
  /* Do we have any more header data to send for this file? */
  if(phs->hdr_index < NUM_FILE_HDR_STRINGS)
  {
      len = SPI_BUFFER_SIZE;    
      while(phs->hdr_index < NUM_FILE_HDR_STRINGS)
      {
          /* How much do we have to send from the current header? */
          hdrlen = strlen(phs->hdrs[phs->hdr_index]);

          /* How much of this can we send? */
          sendlen = (len < (hdrlen - phs->hdr_pos)) ?
                          len : (hdrlen - phs->hdr_pos);

          memcpy(&testBuff[testIdx], (const void *)(phs->hdrs[phs->hdr_index] +
                            phs->hdr_pos), sendlen);
          testIdx+=sendlen;

          /* Fix up the header position for the next time round. */
          phs->hdr_pos += sendlen;
          len -= sendlen;

          /* Have we finished sending this string? */
          if(phs->hdr_pos == hdrlen) {
              /* Yes - move on to the next one */
              phs->hdr_index++;
              phs->hdr_pos = 0;
          }
      }
      
      /* Send this amount of data or as much as we can given memory
       * constraints. */
        ROM_GPIOPinWrite(GPIO_PORTD_BASE, GPIO_PIN_1, GPIO_PIN_1);
        UARTprintf("Sending Socket: %d.\n", ulServerDataSocket);
        rcvdlen = send(ulServerDataSocket, testBuff, testIdx, 0);
        UARTprintf("Sent Socket: %d.\n", ulServerDataSocket);
        ROM_GPIOPinWrite(GPIO_PORTD_BASE, GPIO_PIN_1, 0);
        
        if (rcvdlen > 0) {
          /* Remember that we added some more data to be transmitted. */
          data_to_send = true;
        }
            

      /* If we get here and there are still header bytes to send, we send
       * the header information we just wrote immediately.  If there are no
       * more headers to send, but we do have file data to send, drop through
       * to try to send some file data too.
       */
      if((phs->hdr_index < NUM_FILE_HDR_STRINGS) || !phs->file) {
        return(-1);
      }
  }
#else
  /* Assume no error until we find otherwise */
  err = ESUCCESS;
#endif

  /* Have we run out of file data to send? If so, we need to read the next
   * block from the file.
   */
  if(phs->left == 0)
  {
    int count;

    /* Do we have a valid file handle? */
    if(phs->handle == NULL)
    {
        //
        // No - close the connection.
        //
        close_conn();
        return(-1);
    }

    /* Read a block of data from the file. */
    count = fs_read(phs->handle, pcCC3000_Tx_Buffer, SPI_BUFFER_SIZE);
    if(count < 0) {
      /* We reached the end of the file so this request is done */
      close_conn();
      fs_close(phs->handle);
      phs->handle = NULL;      
      return(-1);
    }

    /* Set up to send the block of data we just read */
    phs->left = count;
    phs->file = phs->buf;
  }

      /* We are not processing an SHTML file so no tag checking is necessary.
       * Just send the data as we received it from the file.
       */

      if (TX_MTU < phs->left)
      {
        len = TX_MTU;
      } else 
      {
        len = phs->left;
      }
      /* If the data is being read from a buffer in RAM, we need to copy it
       * into the PCB. If it's in flash, however, we can avoid the copy since
       * the data is obviously not going to be overwritten during the life
       * of the connection.
       */
      ROM_GPIOPinWrite(GPIO_PORTD_BASE, GPIO_PIN_1, GPIO_PIN_1);
      UARTprintf("Sending Socket: %d.\n", ulServerDataSocket);
      rcvdlen = send(ulServerDataSocket, phs->file, len, 0);
      UARTprintf("Sent Socket: %d.\n", ulServerDataSocket);
      ROM_GPIOPinWrite(GPIO_PORTD_BASE, GPIO_PIN_1, 0);

      if (rcvdlen > 0) {
        data_to_send = true;
        phs->file += rcvdlen;
        phs->left -= rcvdlen;
      }
  /* If we wrote anything to be sent, go ahead and send it now. */
  if(data_to_send) {
    return(0);
  }
  else{
    return(-1);
  }
}


/*-----------------------------------------------------------------------------------*/
static struct fs_file *
get_404_file(char **ppURI)
{
  struct fs_file *file;

  *ppURI = "/404.html";
  file = fs_open(*ppURI);
  if(file == NULL) {
    /* 404.html doesn't exist. Try 404.htm instead. */
    *ppURI = "/404.htm";
    file = fs_open(*ppURI);
    if(file == NULL) {
      /* 404.htm doesn't exist either. Indicate to the caller that it should
       * send back a default 404 page.
       */
      *ppURI = NULL;
    }
  }

  return(file);
}
/*-----------------------------------------------------------------------------------*/
static err_t
http_recv(void)
{
  static unsigned int i;
  signed int iBytesReceived;
  static int loop;
  char *data;
  char *uri;
  struct fs_file *file;
  struct http_state *phs;
#ifdef INCLUDE_HTTPD_CGI
  int count;
  char *params;
#endif 

  // get pointer to file system handle
  phs = &g_hs;
  
  //
  // Get data from client
  //
  UARTprintf("receiving Socket: %d.\n", ulServerDataSocket);
  iBytesReceived = recv(ulServerDataSocket, pcCC3000_Rx_Buffer, 
                        RX_SPI_BUFFER_SIZE, 0);
  UARTprintf("received Socket: %d. Length: %d\n", ulServerDataSocket, iBytesReceived);
  if (iBytesReceived >= 0) 
  {
    if (phs->handle == NULL) {
      data = pcCC3000_Rx_Buffer;
      uri = &data[4];
      if (strncmp(data, "GET ", 4) == 0) {
        /*
         * We have a GET request. Find the end of the URI by looking for the
         * HTTP marker. We can't just use strstr to find this since the request
         * came from an outside source and we can't be sure that it is
         * correctly formed. We need to make sure that our search is bounded
         * by the packet length so we do it manually. If we don't find " HTTP",
         * assume the request is invalid and close the connection.
         */
        for(i = 4; i < (iBytesReceived - 5); i++) {
          if ((data[i] == ' ') && (data[i + 1] == 'H') &&
              (data[i + 2] == 'T') && (data[i + 3] == 'T') &&
              (data[i + 4] == 'P')) {
            data[i] = 0;
            break;
          }
        }
        if(i == (iBytesReceived - 5)) {
          /* We failed to find " HTTP" in the request so assume it is invalid */
          close_conn();
          return(ESUCCESS);
        }

        /*
         * Have we been asked for the default root file?
         */
        if((uri[0] == '/') &&  (uri[1] == 0)) {
          /*
           * Try each of the configured default filenames until we find one
           * that exists.
           */
          for(loop = 0; loop < NUM_DEFAULT_FILENAMES; loop++) {
            file = fs_open((char *)g_psDefaultFilenames[loop].name);
            uri = (char *)g_psDefaultFilenames[loop].name;
            if(file != NULL) {
              break;
            }
          }
          if(file == NULL) {
            /* None of the default filenames exist so send back a 404 page */
            file = get_404_file(&uri);
          }
        } else {
          /* No - we've been asked for a specific file. */
#ifdef INCLUDE_HTTPD_CGI
          /* First, isolate the base URI (without any parameters) */
          params = strchr(uri, '?');
          if(params) {
            *params = '\0';
            params++;
          }

          /* Does the base URI we have isolated correspond to a CGI handler? */
          if(g_iNumCGIs && g_pCGIs) {
            for(i = 0; i < g_iNumCGIs; i++) {
              if(strcmp(uri, g_pCGIs[i].pcCGIName) == 0) {
                /*
                 * We found a CGI that handles this URI so extract the
                 * parameters and call the handler.
                 */
                 count = extract_uri_parameters(phs, params);
                 uri = g_pCGIs[i].pfnCGIHandler(i, count, phs->params,
                                                phs->param_vals);
                 break;
              }
            }

            /* Did we handle this URL as a CGI? If not, reinstate the
             * original URL and pass it to the file system directly. */
            if(i == g_iNumCGIs)
            {
              /* Replace the ? marker at the beginning of the parameters */
              if(params) {
                 params--;
                *params = '?';
              }
            }
          }
#endif
          file = fs_open(uri);
          if(file == NULL) {
            file = get_404_file(&uri);
          }
        }

        if(file) {
          phs->handle = file;
          phs->file = file->data;
          phs->left = file->len;
          phs->retries = 0;
        } else {
          phs->handle = NULL;
          phs->file = NULL;
          phs->left = 0;
          phs->retries = 0;
        }

#ifdef DYNAMIC_HTTP_HEADERS
        /* Determine the HTTP headers to send based on the file extension of
         * the requested URI. */
        get_http_headers(phs, uri);
#endif
        
        /* Start sending the headers and file data. */
        while(send_data() != -1);
        // Return once sending complete
      } 
      else 
      {
        close_conn();
      }
    } 
  }
  else
  {
    return ERR_TIMEOUT;
  }

  return ESUCCESS;
}
/*-----------------------------------------------------------------------------------*/
static void
http_accept(void)
{
  long ulSocketHandleTest;
  ulSocketHandleTest = accept(ulServerSocket, &tClientSocketAddr, &tClientSocketAddrLen);
  
  if(!(ulSocketHandleTest < 0))
  {
    ulServerSocketopen = true;
    UARTprintf("Accept Socket: %d.\n", ulSocketHandleTest);
    UARTprintf("Client: %d.%d.%d.%d\n", tClientSocketAddr.sa_data[2], 
               tClientSocketAddr.sa_data[3], 
               tClientSocketAddr.sa_data[4], 
               tClientSocketAddr.sa_data[5]);
    //
    // Copy over the valid handle.
    //
    ulServerDataSocket = ulSocketHandleTest;
    //
    // We have a client to serve
    //
    struct http_state *phs;
  
    // get pointer to file system handle
    phs = &g_hs;
    /* Initialize the structure. */
    phs->handle = NULL;
    phs->file = NULL;
    phs->buf = NULL;
    phs->buf_len = 0;
    phs->left = 0;
    phs->retries = 0;
  #ifdef DYNAMIC_HTTP_HEADERS
    /* Indicate that the headers are not yet valid */
    phs->hdr_index = NUM_FILE_HDR_STRINGS;
  #endif
    
    http_recv();
  }
  else
  {
    // 
    // accept() returned an error.  Something is wrong
    // so close socket.
    //
    ulServerSocketopen = false;
    closesocket(ulServerSocket);
    UARTprintf("Error Accept Socket: %d. returned: %d.\n", ulServerSocket, ulSocketHandleTest);
  }
}
/*-----------------------------------------------------------------------------------*/
void
httpd_init(void)
{
    long lerr;
    sockaddr tSocketAddr;
    
    if(ulServerSocketopen == false)
    {
      tSocketAddr.sa_family = AF_INET;
    
      //
      // Web HTTP Port
      //
      unsigned long port;
      port = htons(80);
      memcpy(&tSocketAddr.sa_data[0], &port, sizeof(unsigned short));
        
      // IP addr any
      memset(&tSocketAddr.sa_data[2], INADDR_ANY, 4);
          
      ulServerSocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
      UARTprintf("Opened Socket: %d.\n", ulServerSocket);
      lerr = bind(ulServerSocket, &tSocketAddr, sizeof(tSocketAddr));
      UARTprintf("bind Socket: %d. Returned: %d\n", ulServerSocket, lerr);
      if(lerr >= 0)
      {
        lerr = listen(ulServerSocket, 5);
        UARTprintf("listen Socket: %d. Returned: %d\n", ulServerSocket, lerr);
      }
      if(lerr >= 0)
      {
        http_accept();
      }  
    }
    else
    {
      http_accept();
    }
}

long
http_poll_recv(void)
{
    struct timeval timeout;
    fd_set socks;
    static signed int iReadSocks;
    //
	// Timeout set for 1 ms
	//
	timeout.tv_sec = 0;
	timeout.tv_usec = 1000;
    //
	// First use select to determine if there are any messages ready
	// to receive
	//
	FD_ZERO(&socks);
	FD_SET(ulServerDataSocket, &socks);

    //
    // make sure that the socket is open
    // or else the select will block
    //
	if(http_is_socket_open(ulServerDataSocket))
    {
        //
        // First argument is Number of sockets +
        // We are binded to 1 sockets so we pass a 2
        //
        iReadSocks = select(2, &socks, 0, 0, &timeout);
        if (iReadSocks > 0) 
        {
          // Yes there are messages. return true.
          return 1;
        }
    }
    // No messages or the socket is closed.
    return 0;
}
/*-----------------------------------------------------------------------------------*/
long http_is_socket_open(long lSocketHandle)
{
    int cRetVal;
    unsigned long uloptval;
    socklen_t optlen;
    
    // Test Server Socket to see if it is still open
    cRetVal = getsockopt(lSocketHandle, SOL_SOCKET,
            SOCKOPT_RECV_TIMEOUT, &uloptval, &optlen);
    if (cRetVal == -1) 
    {
		return(false);
	}
    return(true);
}
/*-----------------------------------------------------------------------------------*/
long http_is_data_rcvd(long lSocketHandle)
{
    struct timeval timeout;
    fd_set socks;
    static signed int iReadSocks;
    //
    // Timeout set for 1 ms
    //
    timeout.tv_sec = 0;
    timeout.tv_usec = 1000;
    //
    // First use select to determine if there are any messages ready
    // in the CC3000 for the thermostat
    //
    FD_ZERO(&socks);
    FD_SET(lSocketHandle, &socks);
    //
    // First argument is Number of sockets +
    // We are binded to 1 sockets so we pass a 2
    //
    iReadSocks = select(2, &socks, 0, 0, &timeout);
    if (iReadSocks > 0)
    {
        return(true);
	}
  
    return(false);
}

#ifdef INCLUDE_HTTPD_CGI
/*-----------------------------------------------------------------------------------*/
void
http_set_cgi_handlers(const tCGI *pCGIs, int iNumHandlers)
{
    g_pCGIs = pCGIs;
    g_iNumCGIs = iNumHandlers;
}
#endif


static long
ParseXmlTag(char *pFile, char *pcParent, char *pcChild, char **pcOutput, int* iOutputLen)
{
  char *pStr;
  // Scan file for parent section
  pStr = strstr(pFile, pcParent);
  pStr = strstr(pStr, pcChild);
  // offset by 7 as the XML tag is "<pcChild> data="
  *pcOutput = &pStr[7 + strlen(pcChild)];
  // Find the end of the string
  pStr = strchr(*pcOutput, '"');
  *iOutputLen = pStr - *pcOutput;
  
  if(*iOutputLen > 0)
  {
    return ESUCCESS;
  }
  else
  {
    return EFAIL;
  }
}
static int
ParseXmlFile(char * pFile, WeatherForcast_t *ForcastData)
{
  long lerr;
  char *pStr;
  int ilen;
  //
  // parse XML for tags
  //
  // Remove warning
  pStr = NULL;
  // Test if the response is in error. most likely due to a 
  // bad city name.
  lerr = ParseXmlTag(pFile, "<problem_cause", "data", \
    &pStr, &ilen);  
  if(lerr == ESUCCESS)
  {  
    memcpy(ForcastData->pcCityName, "Unknown City", sizeof("unknown City"));
    ForcastData->pcCityName[sizeof("unknown City")] = '\0';    
    return 0;
  }
  
  // Get City
  lerr = ParseXmlTag(pFile, "<forecast_information>", "city", \
    &pStr, &ilen);
  if(lerr == ESUCCESS && pStr != NULL)
  {
    memcpy(ForcastData->pcCityName, pStr, ilen);
    ForcastData->pcCityName[ilen] = '\0';
  }
  else
  {
    return 0;
  }
  // Get Picture
  lerr = ParseXmlTag(pFile, "<current_conditions>", "icon", \
      &pStr, &ilen);
  if(lerr == ESUCCESS)
  {
    memcpy(ForcastData->pcIconUrl, pStr, ilen);
    ForcastData->pcIconUrl[ilen] = '\0';
  }
  else
  {
    return 0;
  }
  // Get Current Temperature
  lerr = ParseXmlTag(pFile, "<current_conditions>", "temp_f", \
    &pStr, &ilen);
  if(lerr == ESUCCESS)
  {
    ForcastData->iTemperatureCurrent = atoi(pStr);
  }
  else
  {
    return 0;
  }
  // Get High and Low Temperature forcast for today
  lerr = ParseXmlTag(pFile, "<forecast_conditions>", "low", \
    &pStr, &ilen);
  if(lerr == ESUCCESS)
  {
    ForcastData->iTemperatureLow = atoi(pStr);
  }
  else
  {
    return 0;
  }
  lerr = ParseXmlTag(pFile, "<forecast_conditions>", "high", \
    &pStr, &ilen);
  if(lerr == ESUCCESS)
  {
    ForcastData->iTemperatureHigh = atoi(pStr);
  }
  else
  {
    return 0;
  }
  return 1;
}

//*****************************************************************************
//
//! http_get_weather_for_city
//!
//!  \param  none
//!
//!  \return none
//!
//!  \brief  This function is the main web client application.  It sets up
//!         a TCP socket to google.com and then performs a restful weather api
//!         request for the city of interest.  The response is an XML file.
//!         The XML file is parsed using known tags.
//
//*****************************************************************************
long 
http_get_weather_for_city(char* pcCityName, WeatherForcast_t *WeatherData)
{
  long lerr;
  sockaddr tSocketAddr;  
  long iBytesReceived;
  long ulClientSocket;
  long SentLen;
  unsigned long ulGooglesIpAddr;
  
  //
  // Resolve Google IP address
  //
  lerr = gethostbyname("www.google.com", 14, &ulGooglesIpAddr);
  
  if(lerr < ESUCCESS)
  {
    //
    // Made up my own error code here to indicate no internet connectivity    
    //
    return -74;
  }
  //
  // Connect to Google
  //  
  tSocketAddr.sa_family = AF_INET;

  //
  // Web HTTP Port
  //
  unsigned long port;
  port = htons(80);
  memcpy(&tSocketAddr.sa_data[0], &port, sizeof(unsigned short));    
  // IP addr Google
  ulGooglesIpAddr = htonl(ulGooglesIpAddr);
  memcpy(&tSocketAddr.sa_data[2], (unsigned char*)&ulGooglesIpAddr, 4);
  // Connect Socket
  ulClientSocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
  lerr = connect(ulClientSocket, &tSocketAddr, sizeof(tSocketAddr));
  UARTprintf("Opened Client Socket: %d. returned: %d\n", ulClientSocket, lerr);
  if(lerr != ESUCCESS)
  {
    return lerr;
  }
  
  //
  // Create API request for RSS feed
  //
  char headerPre[] = {"GET /ig/api?weather="};
  char headerPost[] = {" HTTP/1.1\r\nHost: www.google.com\r\n\r\n"};
  char pucWeatherApiRequest[sizeof(headerPre) + sizeof(headerPost) + 20];
  
  strcpy(pucWeatherApiRequest, headerPre); 
  strcat(pucWeatherApiRequest, pcCityName);
  strcat(pucWeatherApiRequest, headerPost);
  //
  // Send request for weather forcast XML
  UARTprintf("Sending Client Socket: %d.\n", ulClientSocket);
  SentLen = send(ulClientSocket, pucWeatherApiRequest,                \
                 strlen(pucWeatherApiRequest), 0);
  UARTprintf("Sent Client Socket: %d.\n", ulClientSocket);
  
  if(SentLen <= 0)
  {
    return SentLen;
  }
  
  //
  // make sure that the socket is open
  // or else the select will block
  //
  if(!http_is_socket_open(ulClientSocket))
  {
    UARTprintf("Error Client Socket not open: %d.\n", ulClientSocket);
    return ERROR_SOCKET_INACTIVE;
  }
  
  // 
  // See if we received a response. If not, keep trying until timeout.
  // Don't just call the recv function or it will block.  
  //
  unsigned int loop;
  for(loop = 0; loop < 20; loop++)
  {
    //
    // Delay for 100ms
    //
    SysCtlDelay(8000000/10);
    //
    // break out of loop if response received
    //
    if(http_is_data_rcvd(ulClientSocket))
    {
        break;
    }
  }
  //
  // See if we received anything yet
  //
  if(loop == 10)
  {
    closesocket(ulClientSocket);
    UARTprintf("Error Client Socket not open. Closing Socket: %d.\n", ulClientSocket);
    return ERROR_SOCKET_INACTIVE;
  }
  
    //
    // Delay for 100ms before reading to ensure that the packet has been
    // received
    //
    SysCtlDelay(8000000/10);
    // 
    // Fetch XML response
    //   
    UARTprintf("receiving Client Socket: %d.\n", ulClientSocket);
    iBytesReceived = recv(ulClientSocket, pcCC3000_Rx_Buffer, 
                      SPI_BUFFER_SIZE, 0);
    UARTprintf("received Client Socket: %d. Length: %d\n", ulClientSocket, iBytesReceived);
    
    if (iBytesReceived >= 0) 
    {
        ParseXmlFile(pcCC3000_Rx_Buffer, WeatherData); 
    }
    
    // 
    // Flush the receive pipe.  Probably not neccessary since we close the 
    // socket but next Since the served feed can be larger than one CC3000 SPI 
    // transfer it is only courteous to finish what we started.
    //
    while(http_is_data_rcvd(ulClientSocket))
    {
        iBytesReceived = recv(ulClientSocket, pcCC3000_Rx_Buffer, 
                              SPI_BUFFER_SIZE, 0);
    }
    //
    // We have our data so close the socket.
    //
    UARTprintf("Close Client Socket: %d.\n", ulClientSocket);
    closesocket(ulClientSocket);
    
    return ESUCCESS;
}

/*-----------------------------------------------------------------------------------*/
